/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "colors.h"
#include "info.h"


#include <QtCore/qmath.h>


#include <QFile>
#include <QTextStream>
#include <QSettings>
#include <QApplication>


namespace Color {

class YUV{
    const float y,u,v;
public:
    YUV(const QColor& cl):y(cl.red()*0.299+cl.green()*0.587+cl.blue()*0.114),
        u(cl.red()*(-0.16874)+cl.green()*(-0.33126)+cl.blue()*0.5+128),
        v(cl.red()*0.5+cl.green()*(-0.41869)+cl.blue()*(-0.08131)+128){}
    float distance(const YUV& x)const{return qSqrt(qPow(y-x.y,2)*4+qPow(u-x.u,2)+qPow(v-x.v,2));}
};

class RGB{
    const float gamma;
    const float r,g,b;
public:
    RGB(const QColor& cl):gamma(1.4),
        r(qPow(cl.red(),gamma)*0.3),
        g(qPow(cl.green(),gamma)*0.59),
        b(qPow(cl.blue(),gamma)*0.11)
    {}
    float distance(const RGB& x)const{return qSqrt(qPow(r-x.r,2)+qPow(g-x.g,2)+qPow(b-x.b,2));}
};

}


using namespace Color;


ColorPalette::ColorPalette()
{
    QSettings rsettings(QApplication::organizationName(),QApplication::applicationName());
    if (!rsettings.contains("palette"))
        rsettings.setValue("palette", "dmc");

    QString paletteName = rsettings.value("palette").value<QString>();

    open(paletteName);
}



QString ColorPalette::name() {
    QSettings rsettings(QApplication::organizationName(),QApplication::applicationName());
    return rsettings.value("palette").value<QString>();
}



float ColorPalette::distance(QRgb c1,QRgb c2)
{

    return qSqrt(qPow((float)(qRed(c1) - qRed(c2)), 2) +
                qPow((float)(qGreen(c1) - qGreen(c2)), 2) +
                qPow((float)(qBlue(c1) - qBlue(c2)), 2));
}



QRgb ColorPalette::find(const QColor& cl,int pos)const
{    
  QList<QRgb>result;
  int precision=500;
  for(int k=0;k<color.size();k++){
    QColor clDMC(color.at(k)._value);
    float err=YUV(clDMC).distance(YUV(cl));
    //float err=RGB(clDMC).distance(RGB(cl));
    if(err<precision){
      result.prepend(color.at(k)._value);
      precision=int(err);
    }
  }

  if ((result.size() > 0) && (pos < result.size()))
      return result.at(pos);
  else
      return blank();
}



void ColorPalette::open(const QString &name)
{
    QString path = Info::dataPath() + "/system/palette/" + name + ".csc";

    QFile file(path);
    if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        color.clear();
        QTextStream in(&file);
        while (!in.atEnd()) {
            QString colorStr=in.readLine();
            QStringList cl=colorStr.split(";;");
            if (cl.size() > 3) {
                Color colorItem;
                colorItem._number = cl.at(0);
                colorItem._description = cl.at(1);
                cl[3].prepend("FF");
                colorItem._value = cl.at(3).toUInt(0, 16);
                color.append(colorItem);
            }
        }
    }
}
