/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef VIEW_H
#define VIEW_H

#include "history.h"
#include "sprite.h"
#include "line.h"

#include <QWidget>


class QGridLayout;


class View: public QWidget
{
 Q_OBJECT

 public:

  History history;

  class LineMode{
    friend class View;
    enum Mode{Add,Del,No};
    Line           line;
    Mode           mode;
    LineMode(const LineMode& ln):line(ln.line),mode(ln.mode){}
  public:
    LineMode(uchar w=0,uchar m=2):line(),mode((Mode)m){line.setWidth(w);}
  };

   View(QSize size,QWidget *parent=0);

   static QColor invertColor(const QColor& cl);

   int squarePix()const{return _squarePix;}
   int dx()const{return _dx;}
   int spriteSize()const{return sprite.size();}
   int lineSize()const{return line.size();}
   int colorCount(const QColor &cl)const;
   int colorCount(QRgb rgbcl)const;
   void swapColorToBackground(QRgb rgbcl){
     for(int i=0;i<sprite.size();i++)
       if(sprite.at(i)->color()==rgbcl){
         sprite[i]->setColor(_backColor);
         sprite[i]->setIconID(INT_MAX);
       }
     int i=0;
     while(i<line.size()){
       if(line.at(i)->color()==rgbcl)delete line.takeAt(i);
       else i++;
     }
   }

   void clear();
   void setBackground(const QColor &cl);
   void setDL(const QPoint &dl){_dl=dl;}
   void setClPicCount(int cl){_clpicCount=cl;}
   void setFont(const QFont &font){_font=font;}
   void setPen(const QPen &pen){_pen=pen;}
   void setSquarePix(int pix);
   void setParameters(int squarePix,int imgW,int imgH,int dX,int linePix,int dl);
   void setIconSelected(int id){_iconSelected=id;}
   void setDX(int dx){_dx=dx;}
   void setColorSelected(const QColor &cl);
   void setImageList(const QList<QImage> &list){imageList.clear();  imageList.append(list);}
   void colorUpdate(const QColor &clOld,const QColor &clNew);
   void iconUpdate(const QColor &cl,int num);
   bool createSprite(const QPoint &point,const QColor &cl,int id);
   void createLine(const QPoint &p1,const QPoint &p2,const QColor &cl,int w);
   void setLine(const LineMode& ln){linePreview=ln;}
   QImage toImage(bool fastPreview=false);
   QImage toImage(QRect r);
   Sprite* getSprite(int i)const;
   Line* getLine(int i)const;
   ColorItem* colorItem(int row,const QColor& cl);
   IconItem*  iconItem(int row,const QColor& cl);
   qreal scale(){return _scale;}
   QSize schemeSize(){return _size;}
   virtual QSize sizeHint()const{return _squarePix*_size+2*QSize(_dx,_dx);}


 public slots:
   void setScale(qreal factor){
       _scale=factor;
       resize(_scale*sizeHint());
       //update();
   }

   void undo();
   void redo();

   void setStitchCross(){stitch=Sprite::Cross;}
   void setStitchLeft(){stitch=Sprite::Left;}
   void setStitchRight(){stitch=Sprite::Right;}
   void setGrid(bool grid){_grid=grid;  update();}
   void setIcons(bool icons){_icons=icons;  update();}
   void setColors(bool colors) { _colors = colors; update(); }
   void setSpriteIconInvert(bool sii){_spriteIconInvert=sii;  update();}
   void setLinePix(int pix){_linePix=pix;  if(_grid)update();}


 protected:
   void paintEvent(QPaintEvent *event);
   void mousePressEvent(QMouseEvent *event);
   void mouseReleaseEvent(QMouseEvent*);
   void mouseMoveEvent(QMouseEvent *event);
   //void resizeEvent(QResizeEvent * event);


 signals:
   void mousePos(QPoint);
   void colorSelected(QPixmap);
   void updated();
   //void sizeChanged(QRectF);


 private:
   void showGrid(QPainter *surf,QRect r=QRect(),int counterX=0,int counterY=0)const;
   void showColor(QPainter *surf)const;
   void showLines(QPainter *surf,const QPoint& dx=QPoint(0,0))const;
   void setStitchImages(const QSize& size);

   void spriteFill(int i,const QColor &filledColor,const QColor &color);

   void spriteFillLineUp(int i,const QColor &filledColor,const QColor &color,QList<int> *index);
   void spriteFillLineDown(int i,const QColor &filledColor,const QColor &color,QList<int> *index);
   QColor spriteColor(int i);

   void setColor(const QPoint &pos,const QColor &color);
   void spriteFill(const QPoint &pos);
   void spriteFill(int i);
   QPalette transparency();
   QColor getColor(const QPoint &pos)const;


   QPoint linePoint(const QPoint &pos)const;
   int getIndexLine(const Line& ln)const;
   inline int getIndexSprite(const QPointF &pos)const;//Нахождение номера спрайта в массиве по координате
   bool isValidSprite(int i)const{return((i>=0)&&(i<sprite.size()))?true:false;}


   QList<Sprite*>sprite;
   QList<Line*>line;
   QList<QImage>imageList;


   int _linePix,
       _squarePix,
       _iconSelected,
       _lineSelected,
       _dx;//Отступы

   uint _clpicCount;

   bool _grid,
        _icons,
        _colors,
        _spriteIconInvert,
        _spriteLine;

   LineMode linePreview;
   qreal _scale;
   QPoint _dl;//Сдвиг позиции отображения пиктограммы цвета
   QColor _colorSelected,
          _backColor;
   QPen _pen;
   QFont _font;
   QSize _size;//Размер схемы (в спрайтах)
   QGridLayout* _layout;
   QImage rstitch,lstitch,lst,rst;
   Sprite::Stitch stitch;

};

#endif
